@@ -1,5 +1,5 @@
---- sys.stable/net/if_ppp.h	Sat Oct 18 04:02:39 1997
-+++ /usr/src/synclink/bsd3/if_ppp.h	Fri Apr 16 12:54:12 1999
+--- sys.stable/net/if_ppp.h	Fri Apr 16 16:14:37 1999
++++ /usr/src/synclink/bsd2/if_ppp.h	Tue Apr 20 15:30:48 1999
 @@ -53,6 +53,7 @@
  #define SC_RCV_B7_1	0x02000000	/* have rcvd char with bit 7 = 1 */
  #define SC_RCV_EVNP	0x04000000	/* have rcvd char with even parity */
