@@ -1,5 +1,4 @@
-/*	$NetBSD: if_ppp.c,v 1.41 1998/05/02 14:34:24 christos Exp $	*/
-/*	Id: if_ppp.c,v 1.6 1997/03/04 03:33:00 paulus Exp 	*/
+/*	$Id: if_ppp.c,v 1.8 1998/03/25 04:04:37 paulus Exp $	*/
 
 /*
  * if_ppp.c - Point-to-Point Protocol (PPP) Asynchronous driver.
@@ -91,6 +90,10 @@
 #include <sys/time.h>
 #include <sys/malloc.h>
 
+#if NetBSD1_0 && defined(i386)
+#include <machine/psl.h>
+#endif
+
 #include <net/if.h>
 #include <net/if_types.h>
 #include <net/netisr.h>
@@ -121,6 +124,10 @@
 #include <net/if_pppvar.h>
 #include <machine/cpu.h>
 
+#if NetBSD1_0
+#define splsoftnet	splnet
+#endif
+
 #ifdef PPP_COMPRESS
 #define PACKETPTR	struct mbuf *
 #include <net/ppp-comp.h>
@@ -164,10 +171,10 @@ extern struct compressor ppp_bsd_compress;
 extern struct compressor ppp_deflate, ppp_deflate_draft;
 
 struct compressor *ppp_compressors[8] = {
-#if DO_BSD_COMPRESS && defined(PPP_BSDCOMP)
+#if DO_BSD_COMPRESS
     &ppp_bsd_compress,
 #endif
-#if DO_DEFLATE && defined(PPP_DEFLATE)
+#if DO_DEFLATE
     &ppp_deflate,
     &ppp_deflate_draft,
 #endif
@@ -186,9 +193,8 @@ pppattach()
     register int i = 0;
 
     for (sc = ppp_softc; i < NPPP; sc++) {
-	sc->sc_unit = i;	/* XXX */
-	sprintf(sc->sc_if.if_xname, "ppp%d", i++);
-	sc->sc_if.if_softc = sc;
+	sc->sc_if.if_name = "ppp";
+	sc->sc_if.if_unit = i++;
 	sc->sc_if.if_mtu = PPP_MTU;
 	sc->sc_if.if_flags = IFF_POINTOPOINT | IFF_MULTICAST;
 	sc->sc_if.if_type = IFT_PPP;
@@ -204,6 +210,18 @@ pppattach()
 	bpfattach(&sc->sc_bpf, &sc->sc_if, DLT_PPP, PPP_HDRLEN);
 #endif
     }
+
+#if NetBSD1_0 && defined(i386)
+    /*
+     * XXX kludge to fix the bug in the i386 interrupt handling code,
+     * where software interrupts could be taken while hardware
+     * interrupts were blocked.
+     */
+    if ((imask[IPL_TTY] & (1 << SIR_NET)) == 0) {
+	imask[IPL_TTY] |= (1 << SIR_NET);
+	intr_calculatemasks();
+    }
+#endif
 }
 
 /*
@@ -235,7 +253,7 @@ pppalloc(pid)
     MALLOC(sc->sc_comp, struct slcompress *, sizeof(struct slcompress),
 	   M_DEVBUF, M_NOWAIT);
     if (sc->sc_comp)
-	sl_compress_init(sc->sc_comp);
+	sl_compress_init(sc->sc_comp, -1);
 #endif
 #ifdef PPP_COMPRESS
     sc->sc_xc_state = NULL;
@@ -345,7 +363,7 @@ pppioctl(sc, cmd, data, flag, p)
 	break;
 
     case PPPIOCGUNIT:
-	*(int *)data = sc->sc_unit;	/* XXX */
+	*(int *)data = sc->sc_if.if_unit;
 	break;
 
     case PPPIOCGFLAGS:
@@ -384,7 +402,7 @@ pppioctl(sc, cmd, data, flag, p)
 	    return (error);
 	if (sc->sc_comp) {
 	    s = splsoftnet();
-	    sl_compress_setup(sc->sc_comp, *(int *)data);
+	    sl_compress_init(sc->sc_comp, *(int *)data);
 	    splx(s);
 	}
 	break;
@@ -423,8 +441,8 @@ pppioctl(sc, cmd, data, flag, p)
 		    sc->sc_xc_state = (*cp)->comp_alloc(ccp_option, nb);
 		    if (sc->sc_xc_state == NULL) {
 			if (sc->sc_flags & SC_DEBUG)
-			    printf("%s: comp_alloc failed\n",
-				sc->sc_if.if_xname);
+			    printf("ppp%d: comp_alloc failed\n",
+			       sc->sc_if.if_unit);
 			error = ENOBUFS;
 		    }
 		    splimp();
@@ -438,8 +456,8 @@ pppioctl(sc, cmd, data, flag, p)
 		    sc->sc_rc_state = (*cp)->decomp_alloc(ccp_option, nb);
 		    if (sc->sc_rc_state == NULL) {
 			if (sc->sc_flags & SC_DEBUG)
-			    printf("%s: decomp_alloc failed\n",
-				sc->sc_if.if_xname);
+			    printf("ppp%d: decomp_alloc failed\n",
+			       sc->sc_if.if_unit);
 			error = ENOBUFS;
 		    }
 		    splimp();
@@ -449,9 +467,9 @@ pppioctl(sc, cmd, data, flag, p)
 		return (error);
 	    }
 	if (sc->sc_flags & SC_DEBUG)
-	    printf("%s: no compressor for [%x %x %x], %x\n",
-		sc->sc_if.if_xname, ccp_option[0], ccp_option[1],
-		ccp_option[2], nb);
+	    printf("ppp%d: no compressor for [%x %x %x], %x\n",
+		   sc->sc_if.if_unit, ccp_option[0], ccp_option[1],
+		   ccp_option[2], nb);
 	return (EINVAL);	/* no handler found */
 #endif /* PPP_COMPRESS */
 
@@ -539,8 +557,8 @@ pppsioctl(ifp, cmd, data)
     u_long cmd;
     caddr_t data;
 {
-    register struct proc *p = curproc;	/* XXX */
-    register struct ppp_softc *sc = ifp->if_softc;
+    struct proc *p = curproc;	/* XXX */
+    register struct ppp_softc *sc = &ppp_softc[ifp->if_unit];
     register struct ifaddr *ifa = (struct ifaddr *)data;
     register struct ifreq *ifr = (struct ifreq *)data;
     struct ppp_stats *psp;
@@ -639,7 +657,7 @@ pppoutput(ifp, m0, dst, rtp)
     struct sockaddr *dst;
     struct rtentry *rtp;
 {
-    register struct ppp_softc *sc = ifp->if_softc;
+    register struct ppp_softc *sc = &ppp_softc[ifp->if_unit];
     int protocol, address, control;
     u_char *cp;
     int s, error;
@@ -683,7 +701,7 @@ pppoutput(ifp, m0, dst, rtp)
 	mode = NPMODE_PASS;
 	break;
     default:
-	printf("%s: af%d not supported\n", ifp->if_xname, dst->sa_family);
+	printf("ppp%d: af%d not supported\n", ifp->if_unit, dst->sa_family);
 	error = EAFNOSUPPORT;
 	goto bad;
     }
@@ -726,7 +744,7 @@ pppoutput(ifp, m0, dst, rtp)
 	len += m->m_len;
 
     if (sc->sc_flags & SC_LOG_OUTPKT) {
-	printf("%s output: ", ifp->if_xname);
+	printf("ppp%d output: ", ifp->if_unit);
 	pppdumpm(m0);
     }
 
@@ -754,7 +772,7 @@ pppoutput(ifp, m0, dst, rtp)
 	*mtod(m0, u_char *) = address;
 #else
 	/*
-	 * Update the time we sent the most recent packet.
+	 * Update the time we sent the most recent data packet.
 	 */
 	sc->sc_last_sent = time.tv_sec;
 #endif /* PPP_FILTER */
@@ -1067,7 +1085,7 @@ ppp_ccp(sc, m, rcvd)
     if (dp + slen > ep) {
 	if (sc->sc_flags & SC_DEBUG)
 	    printf("if_ppp/ccp: not enough data in mbuf (%p+%x > %p+%x)\n",
-		dp, slen, mtod(mp, u_char *), mp->m_len);
+		   dp, slen, mtod(mp, u_char *), mp->m_len);
 	return;
     }
 
@@ -1092,7 +1110,7 @@ ppp_ccp(sc, m, rcvd)
 		if (sc->sc_xc_state != NULL
 		    && (*sc->sc_xcomp->comp_init)
 			(sc->sc_xc_state, dp + CCP_HDRLEN, slen - CCP_HDRLEN,
-			 sc->sc_unit, 0, sc->sc_flags & SC_DEBUG)) {
+			 sc->sc_if.if_unit, 0, sc->sc_flags & SC_DEBUG)) {
 		    s = splimp();
 		    sc->sc_flags |= SC_COMP_RUN;
 		    splx(s);
@@ -1102,7 +1120,7 @@ ppp_ccp(sc, m, rcvd)
 		if (sc->sc_rc_state != NULL
 		    && (*sc->sc_rcomp->decomp_init)
 			(sc->sc_rc_state, dp + CCP_HDRLEN, slen - CCP_HDRLEN,
-			 sc->sc_unit, 0, sc->sc_mru,
+			 sc->sc_if.if_unit, 0, sc->sc_mru,
 			 sc->sc_flags & SC_DEBUG)) {
 		    s = splimp();
 		    sc->sc_flags |= SC_DECOMP_RUN;
@@ -1196,7 +1214,7 @@ ppp_inproc(sc, m)
 	ilen = 0;
 	for (mp = m; mp != NULL; mp = mp->m_next)
 	    ilen += mp->m_len;
-	printf("%s: got %d bytes\n", ifp->if_xname, ilen);
+	printf("ppp%d: got %d bytes\n", ifp->if_unit, ilen);
 	pppdumpm(m);
     }
 
@@ -1238,7 +1256,7 @@ ppp_inproc(sc, m)
 	     * CCP down or issue a Reset-Req.
 	     */
 	    if (sc->sc_flags & SC_DEBUG)
-		printf("%s: decompress failed %d\n", ifp->if_xname, rv);
+		printf("ppp%d: decompress failed %d\n", ifp->if_unit, rv);
 	    s = splimp();
 	    sc->sc_flags |= SC_VJ_RESET;
 	    if (rv == DECOMP_ERROR)
@@ -1288,8 +1306,8 @@ ppp_inproc(sc, m)
 
 	if (xlen <= 0) {
 	    if (sc->sc_flags & SC_DEBUG)
-		printf("%s: VJ uncompress failed on type comp\n",
-		    ifp->if_xname);
+		printf("ppp%d: VJ uncompress failed on type comp\n",
+			ifp->if_unit);
 	    goto bad;
 	}
 
@@ -1340,8 +1358,8 @@ ppp_inproc(sc, m)
 
 	if (xlen < 0) {
 	    if (sc->sc_flags & SC_DEBUG)
-		printf("%s: VJ uncompress failed on type uncomp\n",
-		    ifp->if_xname);
+		printf("ppp%d: VJ uncompress failed on type uncomp\n",
+			ifp->if_unit);
 	    goto bad;
 	}
 
@@ -1438,7 +1456,7 @@ ppp_inproc(sc, m)
 	IF_DROP(inq);
 	splx(s);
 	if (sc->sc_flags & SC_DEBUG)
-	    printf("%s: input queue full\n", ifp->if_xname);
+	    printf("ppp%d: input queue full\n", ifp->if_unit);
 	ifp->if_iqdrops++;
 	goto bad;
     }
