@@ -1,5 +1,4 @@
-/*	$NetBSD: ppp_tty.c,v 1.13 1997/03/25 22:33:25 christos Exp $	*/
-/*	Id: ppp_tty.c,v 1.3 1996/07/01 01:04:11 paulus Exp 	*/
+/*	$Id: ppp_tty.c,v 1.4 1997/04/30 05:47:38 paulus Exp $	*/
 
 /*
  * ppp_tty.c - Point-to-Point Protocol (PPP) driver for asynchronous
@@ -373,7 +372,7 @@ pppwrite(tp, uio, flag)
     bcopy(mtod(m0, u_char *), dst.sa_data, PPP_HDRLEN);
     m0->m_data += PPP_HDRLEN;
     m0->m_len -= PPP_HDRLEN;
-    return ((*sc->sc_if.if_output)(&sc->sc_if, m0, &dst, (struct rtentry *)0));
+    return (pppoutput(&sc->sc_if, m0, &dst, (struct rtentry *)0));
 }
 
 /*
@@ -801,7 +800,7 @@ pppinput(c, tp)
     if (c & TTY_FE) {
 	/* framing error or overrun on this char - abort packet */
 	if (sc->sc_flags & SC_DEBUG)
-	    printf("%s: bad char %x\n", sc->sc_if.if_xname, c);
+	    printf("ppp%d: bad char %x\n", sc->sc_if.if_unit, c);
 	goto flush;
     }
 
@@ -857,8 +856,8 @@ pppinput(c, tp)
 	    sc->sc_flags |= SC_PKTLOST;	/* note the dropped packet */
 	    if ((sc->sc_flags & (SC_FLUSH | SC_ESCAPED)) == 0){
 		if (sc->sc_flags & SC_DEBUG)
-		    printf("%s: bad fcs %x\n", sc->sc_if.if_xname,
-			sc->sc_fcs);
+		    printf("ppp%d: bad fcs %x, pkt len %d\n",
+			   sc->sc_if.if_unit, sc->sc_fcs, ilen);
 		sc->sc_if.if_ierrors++;
 		sc->sc_stats.ppp_ierrors++;
 	    } else
@@ -870,7 +869,7 @@ pppinput(c, tp)
 	if (ilen < PPP_HDRLEN + PPP_FCSLEN) {
 	    if (ilen) {
 		if (sc->sc_flags & SC_DEBUG)
-		    printf("%s: too short (%d)\n", sc->sc_if.if_xname, ilen);
+		    printf("ppp%d: too short (%d)\n", sc->sc_if.if_unit, ilen);
 		s = spltty();
 		sc->sc_if.if_ierrors++;
 		sc->sc_stats.ppp_ierrors++;
@@ -942,7 +941,7 @@ pppinput(c, tp)
 	    pppgetm(sc);
 	    if (sc->sc_m == NULL) {
 		if (sc->sc_flags & SC_DEBUG)
-		    printf("%s: no input mbufs!\n", sc->sc_if.if_xname);
+		    printf("ppp%d: no input mbufs!\n", sc->sc_if.if_unit);
 		goto flush;
 	    }
 	}
@@ -955,8 +954,8 @@ pppinput(c, tp)
 	if (c != PPP_ALLSTATIONS) {
 	    if (sc->sc_flags & SC_REJ_COMP_AC) {
 		if (sc->sc_flags & SC_DEBUG)
-		    printf("%s: garbage received: 0x%x (need 0xFF)\n",
-		    sc->sc_if.if_xname, c);
+		    printf("ppp%d: garbage received: 0x%x (need 0xFF)\n",
+			   sc->sc_if.if_unit, c);
 		goto flush;
 	    }
 	    *sc->sc_mp++ = PPP_ALLSTATIONS;
@@ -967,8 +966,8 @@ pppinput(c, tp)
     }
     if (sc->sc_ilen == 1 && c != PPP_UI) {
 	if (sc->sc_flags & SC_DEBUG)
-	    printf("%s: missing UI (0x3), got 0x%x\n",
-		sc->sc_if.if_xname, c);
+	    printf("ppp%d: missing UI (0x3), got 0x%x\n",
+		   sc->sc_if.if_unit, c);
 	goto flush;
     }
     if (sc->sc_ilen == 2 && (c & 1) == 1) {
@@ -979,15 +978,15 @@ pppinput(c, tp)
     }
     if (sc->sc_ilen == 3 && (c & 1) == 0) {
 	if (sc->sc_flags & SC_DEBUG)
-	    printf("%s: bad protocol %x\n", sc->sc_if.if_xname,
-		(sc->sc_mp[-1] << 8) + c);
+	    printf("ppp%d: bad protocol %x\n", sc->sc_if.if_unit,
+		   (sc->sc_mp[-1] << 8) + c);
 	goto flush;
     }
 
     /* packet beyond configured mru? */
     if (++sc->sc_ilen > sc->sc_mru + PPP_HDRLEN + PPP_FCSLEN) {
 	if (sc->sc_flags & SC_DEBUG)
-	    printf("%s: packet too big\n", sc->sc_if.if_xname);
+	    printf("ppp%d: packet too big\n", sc->sc_if.if_unit);
 	goto flush;
     }
 
@@ -998,7 +997,7 @@ pppinput(c, tp)
 	    pppgetm(sc);
 	    if (m->m_next == NULL) {
 		if (sc->sc_flags & SC_DEBUG)
-		    printf("%s: too few input mbufs!\n", sc->sc_if.if_xname);
+		    printf("ppp%d: too few input mbufs!\n", sc->sc_if.if_unit);
 		goto flush;
 	    }
 	}
@@ -1037,7 +1036,7 @@ ppplogchar(sc, c)
 	sc->sc_rawin[sc->sc_rawin_count++] = c;
     if (sc->sc_rawin_count >= sizeof(sc->sc_rawin)
 	|| (c < 0 && sc->sc_rawin_count > 0)) {
-	printf("%s input: ", sc->sc_if.if_xname);
+	printf("ppp%d input: ", sc->sc_if.if_unit);
 	pppdumpb(sc->sc_rawin, sc->sc_rawin_count);
 	sc->sc_rawin_count = 0;
     }
