@@ -1,4 +1,5 @@
-/*	$Id: if_pppvar.h,v 1.4 1997/04/30 05:47:02 paulus Exp $	*/
+/*	$NetBSD: if_pppvar.h,v 1.8 1998/02/09 17:43:52 perry Exp $	*/
+/*	Id: if_pppvar.h,v 1.3 1996/07/01 01:04:37 paulus Exp	 */
 
 /*
  * if_pppvar.h - private structures and declarations for PPP.
@@ -42,6 +43,9 @@
  * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
+#ifndef _NET_IF_PPPVAR_H_
+#define _NET_IF_PPPVAR_H_
+
 /*
  * Supported network protocols.  These values are used for
  * indexing sc_npmode.
@@ -54,6 +58,7 @@
  */
 struct ppp_softc {
 	struct	ifnet sc_if;		/* network-visible interface */
+	int	sc_unit;		/* XXX unit number */
 	u_int	sc_flags;		/* control/status bits; see if_ppp.h */
 	void	*sc_devp;		/* pointer to device-dep structure */
 	void	(*sc_start) __P((struct ppp_softc *));	/* start output proc */
@@ -111,3 +116,5 @@ struct	mbuf *ppp_dequeue __P((struct ppp_softc *sc));
 int	pppoutput __P((struct ifnet *, struct mbuf *,
 		       struct sockaddr *, struct rtentry *));
 #endif /* _KERNEL */
+
+#endif /* _NET_IF_PPPVAR_H_ */
