@@ -18,9 +18,7 @@
  * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#ifndef lint
-static char rcsid[] = "$Id: cbcp.c,v 1.6 1999/03/16 22:54:38 paulus Exp $";
-#endif
+#define RCSID	"$Id: cbcp.c,v 1.10 1999/08/13 06:46:10 paulus Exp $"
 
 #include <stdio.h>
 #include <string.h>
@@ -31,7 +29,8 @@ static char rcsid[] = "$Id: cbcp.c,v 1.6 1999/03/16 22:54:38 paulus Exp $";
 #include "cbcp.h"
 #include "fsm.h"
 #include "lcp.h"
-#include "ipcp.h"
+
+static const char rcsid[] = RCSID;
 
 /*
  * Options.
@@ -384,7 +383,7 @@ cbcp_resp(us)
 	PUTCHAR(len , bufp);
 	PUTCHAR(0, bufp);
 	cbcp_send(us, CBCP_RESP, buf, len);
-	(*ipcp_protent.open)(us->us_unit);
+	start_networks();
 	return;
     }
 }
@@ -439,6 +438,8 @@ cbcp_recvack(us, pckt, len)
 	    if (address[0])
 	        dbglog("peer will call: %s", address);
 	}
+	if (type == CB_CONF_NO)
+	    return;
     }
 
     cbcp_up(us);
@@ -451,4 +452,5 @@ cbcp_up(us)
 {
     persist = 0;
     lcp_close(0, "Call me back, please");
+    status = EXIT_CALLBACK;
 }
