@@ -25,9 +25,7 @@
  * OR MODIFICATIONS.
  */
 
-#ifndef lint
-static char rcsid[] = "$Id: ccp.c,v 1.27 1999/03/19 01:19:30 paulus Exp $";
-#endif
+#define RCSID	"$Id: ccp.c,v 1.29 1999/08/13 06:46:11 paulus Exp $"
 
 #include <stdlib.h>
 #include <string.h>
@@ -37,6 +35,8 @@ static char rcsid[] = "$Id: ccp.c,v 1.27 1999/03/19 01:19:30 paulus Exp $";
 #include "ccp.h"
 #include <net/ppp-comp.h>
 
+static const char rcsid[] = RCSID;
+
 /*
  * Command-line options.
  */
