@@ -33,7 +33,7 @@
  * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#define RCSID	"$Id: chap.c,v 1.23 1999/08/13 06:46:11 paulus Exp $"
+#define RCSID	"$Id: chap.c,v 1.24 1999/11/15 01:51:50 paulus Exp $"
 
 /*
  * TODO:
@@ -741,7 +741,7 @@ ChapGenChallenge(cstate)
 {
     int chal_len;
     u_char *ptr = cstate->challenge;
-    unsigned int i;
+    int i;
 
     /* pick a random challenge length between MIN_CHALLENGE_LENGTH and 
        MAX_CHALLENGE_LENGTH */  
@@ -753,7 +753,7 @@ ChapGenChallenge(cstate)
     cstate->chal_transmits = 0;
 
     /* generate a random string */
-    for (i = 0; i < chal_len; i++ )
+    for (i = 0; i < chal_len; i++)
 	*ptr++ = (char) (drand48() * 0xff);
 }
 
