@@ -17,9 +17,7 @@
  * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#ifndef lint
-static char rcsid[] = "$Id: demand.c,v 1.10 1999/03/19 04:23:38 paulus Exp $";
-#endif
+#define RCSID	"$Id: demand.c,v 1.12 1999/08/13 06:46:12 paulus Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -45,6 +43,8 @@ static char rcsid[] = "$Id: demand.c,v 1.10 1999/03/19 04:23:38 paulus Exp $";
 #include "ipcp.h"
 #include "lcp.h"
 
+static const char rcsid[] = RCSID;
+
 char *frame;
 int framelen;
 int framemax;
