@@ -19,9 +19,7 @@
  * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#ifndef lint
-static char rcsid[] = "$Id: sys-NeXT.c,v 1.17 1999/03/22 05:55:36 paulus Exp $";
-#endif
+#define RCSID	"$Id: sys-NeXT.c,v 1.20 1999/08/13 06:46:17 paulus Exp $"
 
 #include <stdio.h>
 #include <termios.h>
@@ -58,6 +56,8 @@ static char rcsid[] = "$Id: sys-NeXT.c,v 1.17 1999/03/22 05:55:36 paulus Exp $";
 
 #include "pppd.h"
 
+static const char rcsid[] = RCSID;
+
 static int initdisc = -1;	/* Initial TTY discipline */
 static int initfdflags = -1;	/* Initial file descriptor flags for fd */
 static int ppp_fd = -1;		/* fd which is set to PPP discipline */
@@ -73,8 +73,6 @@ extern int errno;
 static int	restore_term;	/* 1 => we've munged the terminal */
 static struct termios inittermios; /* Initial TTY termios */
 
-static char *lock_file;
-
 static int sockfd;		/* socket for doing interface ioctls */
 static int pppdev;  /* +++ */
 
@@ -1178,7 +1176,7 @@ int have_route_to(u_int32_t addr)
     return -1;
 }
 
-
+#if 0
 /*
  * daemon - Detach us from the terminal session.
  */
@@ -1202,7 +1200,7 @@ daemon(nochdir, noclose)
     }
     return 0;
 }
-
+#endif
 
 char *
 strdup(s)
@@ -1246,9 +1244,9 @@ logwtmp(line, name, host)
     if ((fd = open(WTMPFILE, O_WRONLY|O_APPEND, 0)) < 0)
 	return;
     if (!fstat(fd, &buf)) {
-	strlcpy(ut.ut_line, line, sizeof(ut.ut_line));
-	strlcpy(ut.ut_name, name, sizeof(ut.ut_name));
-	strlcpy(ut.ut_host, host, sizeof(ut.ut_host));
+	strncpy(ut.ut_line, line, sizeof(ut.ut_line));
+	strncpy(ut.ut_name, name, sizeof(ut.ut_name));
+	strncpy(ut.ut_host, host, sizeof(ut.ut_host));
 	(void)time(&ut.ut_time);
 	if (write(fd, (char *)&ut, sizeof(struct utmp)) != sizeof(struct utmp))
 	    (void)ftruncate(fd, buf.st_size);
@@ -1256,6 +1254,7 @@ logwtmp(line, name, host)
     close(fd);
 }
 
+#if 0
 /*
  * Routines for locking and unlocking the serial device, moved here
  * from chat.c.
@@ -1263,6 +1262,8 @@ logwtmp(line, name, host)
 
 #define LOCK_PREFIX	"/usr/spool/uucp/LCK/LCK.."
 
+static char *lock_file;
+
 /*
  * lock - create a lock file for the named device
  */
@@ -1331,6 +1332,7 @@ unlock()
 	lock_file = NULL;
     }
 }
+#endif
 
 #if defined(i386) && defined(HAS_BROKEN_IOCTL)
 int
