@@ -20,8 +20,7 @@
  * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#ifndef lint
-static char rcsid[] = "$Id: sys-bsd.c,v 1.42 1999/04/01 07:20:10 paulus Exp $";
+#define RCSID	"$Id: sys-bsd.c,v 1.46 1999/08/13 06:46:18 paulus Exp $"
 /*	$NetBSD: sys-bsd.c,v 1.1.1.3 1997/09/26 18:53:04 christos Exp $	*/
 #endif
 
@@ -74,6 +73,8 @@ static char rcsid[] = "$Id: sys-bsd.c,v 1.42 1999/04/01 07:20:10 paulus Exp $";
 #include "fsm.h"
 #include "ipcp.h"
 
+static const char rcsid[] = RCSID;
+
 static int initdisc = -1;	/* Initial TTY discipline for ppp_fd */
 static int initfdflags = -1;	/* Initial file descriptor flags for ppp_fd */
 static int ppp_fd = -1;		/* fd which is set to PPP discipline */
@@ -83,8 +84,6 @@ static int restore_term;	/* 1 => we've munged the terminal */
 static struct termios inittermios; /* Initial TTY termios */
 static struct winsize wsinfo;	/* Initial window size info */
 
-static char *lock_file;		/* name of lock file created */
-
 static int loop_slave = -1;
 static int loop_master;
 static char loop_name[20];
@@ -229,13 +228,13 @@ establish_ppp(fd)
 	/*
 	 * Find out which interface we were given.
 	 */
-	if (ioctl(fd, PPPIOCGUNIT, &ifunit) < 0) {    
+	if (ioctl(fd, PPPIOCGUNIT, &ifunit) < 0)
 	    fatal("ioctl(PPPIOCGUNIT): %m");
     } else {
 	/*
 	 * Check that we got the same unit again.
 	 */
-	if (ioctl(fd, PPPIOCGUNIT, &x) < 0) { 
+	if (ioctl(fd, PPPIOCGUNIT, &x) < 0)
 	    fatal("ioctl(PPPIOCGUNIT): %m");
 	if (x != ifunit)
 	    fatal("transfer_ppp failed: wanted unit %d, got %d", ifunit, x);
@@ -714,6 +713,7 @@ ppp_send_config(unit, mtu, asyncmap, pcomp, accomp)
 	fatal("ioctl (PPPIOCGFLAGS): %m");
     x = pcomp? x | SC_COMP_PROT: x &~ SC_COMP_PROT;
     x = accomp? x | SC_COMP_AC: x &~ SC_COMP_AC;
+    x = sync_serial ? x | SC_SYNC : x & ~SC_SYNC;
     if (ioctl(ppp_fd, PPPIOCSFLAGS, (caddr_t) &x) < 0)
 	fatal("ioctl(PPPIOCSFLAGS): %m");
 }
@@ -835,7 +835,7 @@ get_ppp_stats(u, stats)
     struct ifpppstatsreq req;
 
     memset (&req, 0, sizeof (req));
-    strlcpy(req.ifr_name, interface, sizeof(req.ifr_name));
+    strlcpy(req.ifr_name, ifname, sizeof(req.ifr_name));
     if (ioctl(sockfd, SIOCGPPPSTATS, &req) < 0) {
 	error("Couldn't get PPP statistics: %m");
 	return 0;
@@ -1447,11 +1447,14 @@ get_host_seed()
     return gethostid();
 }
 
+#if 0
 /*
  * lock - create a lock file for the named lock device
  */
 #define	LOCK_PREFIX	"/var/spool/lock/LCK.."
 
+static char *lock_file;		/* name of lock file created */
+
 int
 lock(dev)
     char *dev;
@@ -1521,3 +1524,4 @@ unlock()
 	lock_file = NULL;
     }
 }
+#endif
