@@ -17,7 +17,7 @@
  * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#define RCSID	"$Id: upap.c,v 1.21 1999/09/11 12:09:00 paulus Exp $"
+#define RCSID	"$Id: upap.c,v 1.23 1999/11/20 05:11:47 paulus Exp $"
 
 /*
  * TODO:
@@ -31,7 +31,7 @@
 
 static const char rcsid[] = RCSID;
 
-static bool hide_password;
+static bool hide_password = 1;
 
 /*
  * Command-line options.
@@ -39,6 +39,8 @@ static bool hide_password;
 static option_t pap_option_list[] = {
     { "hide-password", o_bool, &hide_password,
       "Don't output passwords to log", 1 },
+    { "show-password", o_bool, &hide_password,
+      "Show password string in debug log messages", 0 },
     { "pap-restart", o_int, &upap[0].us_timeouttime,
       "Set retransmit timeout for PAP" },
     { "pap-max-authreq", o_int, &upap[0].us_maxtransmits,
@@ -372,7 +374,7 @@ upap_rauthreq(u, inp, id, len)
     /*
      * Parse user/passwd.
      */
-    if (len < sizeof (u_char)) {
+    if (len < 1) {
 	UPAPDEBUG(("pap_rauth: rcvd short packet."));
 	return;
     }
@@ -435,7 +437,7 @@ upap_rauthack(u, inp, id, len)
     /*
      * Parse message.
      */
-    if (len < sizeof (u_char)) {
+    if (len < 1) {
 	UPAPDEBUG(("pap_rauthack: ignoring missing msg-length."));
     } else {
 	GETCHAR(msglen, inp);
@@ -475,7 +477,7 @@ upap_rauthnak(u, inp, id, len)
     /*
      * Parse message.
      */
-    if (len < sizeof (u_char)) {
+    if (len < 1) {
 	UPAPDEBUG(("pap_rauthnak: ignoring missing msg-length."));
     } else {
 	GETCHAR(msglen, inp);
